/*    
    th_dnh_archiver
    Copyright (C) 2010-2011 Lymia <lymiahugs@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package lymia.th_dnh.archiver;

import java.io.File;
import java.io.IOException;

public class ConsoleInterface {
	public static void main(String args[]) {
		Common.startMessage();
		if(args.length!=3) {
			usage();
			return;
		}
		
		String mode = args[0];
		String input = args[1];
		String output = args[2];
		
		if(mode.length()!=1) {
			usage();
			return;
		}
		
		switch(mode.charAt(0)) {
			case 'x': try {
				System.out.println("Extracting...");
				DanmakufuArchive a = new DanmakufuArchive(new File(input));
				a.readEntryList();
				a.extract(new File(output));
				System.out.println("Done!");
			} catch(IOException e) {
				System.out.println("IO error: "+e.getMessage());
				e.printStackTrace();
			} catch(DanmakufuException e) {
				System.out.println("Invalid archive: "+e.getMessage());
				e.printStackTrace();
			}
			break;
			case 'c': case 'p': try {
				System.out.println(mode.charAt(0)=='c'?"Compressing...":"Packing...");
				File inputFile = new File(input);
				DanmakufuArchive a = new DanmakufuArchive(new File(output));
				a.addEntryFromPath(inputFile.getName(), inputFile);
				a.writeArchive(mode.charAt(0)=='c');
				System.out.println("Done!");
			} catch(IOException e) {
				System.out.println("IO error: "+e.getMessage());
				e.printStackTrace();
			}
			break;
			default:
				usage();
				break;
		}
	}
	private static void usage() {
		System.out.println("Usage: [mode] [input] [output]");
		System.out.println("Modes: x - extract, c - create, p - pack");
	}
}
