/*    
    th_dnh_archiver
    Copyright (C) 2010-2011 Lymia <lymiahugs@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package lymia.th_dnh.archiver;

import java.awt.BorderLayout;
import java.awt.GraphicsEnvironment;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.SpringLayout;
import javax.swing.UIManager;

public class GraphicalInterface implements ActionListener {
	private JFrame frame;
	
	private JLabel status = new JLabel("Ready");
	
	private JButton extract  = makeButton("Extract" ,"x");
	private JButton compress = makeButton("Compress","c");
	private JButton pack     = makeButton("Pack"    ,"p");

	private JTextField extractSource  = makeField();
	private JTextField extractDest    = makeField();
	private JTextField compressSource = makeField();
	private JTextField compressDest   = makeField();
	private JTextField packSource     = makeField();
	private JTextField packDest       = makeField();

	private Object sync = new Object();
	
	private GraphicalInterface() {
		frame = new JFrame(Common.NAME_STRING);
		frame.setLayout(new BorderLayout());
		
		JTabbedPane tp = new JTabbedPane();
		frame.add(tp,BorderLayout.CENTER);
		frame.add(status,BorderLayout.SOUTH);
		
		/*scope*/ {
			JPanel pane = new JPanel();
			pane.setLayout(new BorderLayout());
			
			JPanel selector = new JPanel();
			selector.setLayout(new SpringLayout());
			selector.add(new JLabel("Source:"));
			selector.add(extractSource);
			selector.add(makeButton("Browse","b0"));
			selector.add(new JLabel("Target:"));
			selector.add(extractDest);
			selector.add(makeButton("Browse","b1"));
			
	        SpringUtilities.makeCompactGrid(selector,
                    2, 3,
                    1, 1,  
                    1, 1); 

			
			pane.add(selector,BorderLayout.CENTER);
			pane.add(extract,BorderLayout.SOUTH);
			
			tp.addTab("Extract", pane);
		}
		/*scope*/ {
			JPanel pane = new JPanel();
			pane.setLayout(new BorderLayout());
			
			JPanel selector = new JPanel();
			selector.setLayout(new SpringLayout());
			selector.add(new JLabel("Source:"));
			selector.add(compressSource);
			selector.add(makeButton("Browse","b2"));
			selector.add(new JLabel("Target:"));
			selector.add(compressDest);
			selector.add(makeButton("Browse","b3"));
			
	        SpringUtilities.makeCompactGrid(selector,
                    2, 3,
                    1, 1,  
                    1, 1); 
			
			pane.add(selector,BorderLayout.CENTER);
			pane.add(compress,BorderLayout.SOUTH);
			
			tp.addTab("Compress", pane);
		}
		/*scope*/ {
			JPanel pane = new JPanel();
			pane.setLayout(new BorderLayout());
			
			JPanel selector = new JPanel();
			selector.setLayout(new SpringLayout());
			selector.add(new JLabel("Source:"));
			selector.add(packSource);
			selector.add(makeButton("Browse","b4"));
			selector.add(new JLabel("Target:"));
			selector.add(packDest);
			selector.add(makeButton("Browse","b5"));
			
	        SpringUtilities.makeCompactGrid(selector,
                    2, 3,
                    1, 1,  
                    1, 1); 
			
			pane.add(selector,BorderLayout.CENTER);
			pane.add(pack,BorderLayout.SOUTH);
			
			tp.addTab("Pack", pane);
		}
		
		frame.setResizable(false);
		frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		
		frame.pack();
		frame.setLocationByPlatform(true);
		frame.setVisible(true);
	}
	private JButton makeButton(String text, String command) {
		JButton button = new JButton(text);
		button.setActionCommand(command);
		button.addActionListener(this);
		return button;
	}
	private JTextField makeField() {
		JTextField field = new JTextField("<no file selected>");
		field.setEditable(false);
		return field;
	}
	
	private void enterAction() {
		compress.setEnabled(false);
		extract.setEnabled(false);
		frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
	}
	private void exitAction() {
		compress.setEnabled(true);
		extract.setEnabled(true);
		frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
	}
	
	public void actionPerformed(final ActionEvent v) {
		Thread t = new Thread() {
			public void run() {
				synchronized(sync) {
					String command = v.getActionCommand();
					switch(command.charAt(0)) {
						case 'x': try {	
								String source = extractSource.getText();
								String dest   = extractDest.getText();
								
								if(source.equals("<no file selected>")||
								   dest  .equals("<no file selected>")) return;
						
								enterAction();

								status("Extracting...");
								DanmakufuArchive a = new DanmakufuArchive(new File(source));
								a.readEntryList();
								a.extract(new File(dest));
								status("Done!");
							} catch(IOException e) {
								error("IO error: "+e.getMessage());
								e.printStackTrace();
							} catch(DanmakufuException e) {
								error("Invalid archive: "+e.getMessage());
								e.printStackTrace();
							} finally {
								exitAction();
							}
							break;
						case 'c': try {
								String source = compressSource.getText();
								String dest   = compressDest.getText();
								
								if(source.equals("<no file selected>")||
								   dest  .equals("<no file selected>")) return;

								enterAction();
								
								status("Compressing...");
								File inputFile = new File(source);
								DanmakufuArchive a = new DanmakufuArchive(new File(dest));
								a.addEntryFromPath(inputFile.getName(), inputFile);
								a.writeArchive();
								status("Done!");
							} catch(IOException e) {
								error("IO error: "+e.getMessage());
								e.printStackTrace();
							} finally {
								exitAction();
							}
							break;
						case 'p': try {
								String source = packSource.getText();
								String dest   = packDest.getText();
								
								if(source.equals("<no file selected>")||
								   dest  .equals("<no file selected>")) return;
	
								enterAction();
								
								status("Packing...");
								File inputFile = new File(source);
								DanmakufuArchive a = new DanmakufuArchive(new File(dest));
								a.addEntryFromPath(inputFile.getName(), inputFile);
								a.writeArchive(false);
								status("Done!");
							} catch(IOException e) {
								error("IO error: "+e.getMessage());
								e.printStackTrace();
							} finally {
								exitAction();
							}
							break;
						case 'b':
							JFileChooser chooser = new JFileChooser();
							chooser.setMultiSelectionEnabled(false);
							switch(command.charAt(1)) {
								case '0': //extract source
									if(chooser.showOpenDialog(frame)==JFileChooser.APPROVE_OPTION)
										extractSource.setText(chooser.getSelectedFile().getPath());
									break;
								case '1': //extract dest;
									chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
									if(chooser.showSaveDialog(frame)==JFileChooser.APPROVE_OPTION)
										extractDest.setText(chooser.getSelectedFile().getPath());
									break;
								case '2': //compress source
									chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
									if(chooser.showOpenDialog(frame)==JFileChooser.APPROVE_OPTION)
										compressSource.setText(chooser.getSelectedFile().getPath());
									break;
								case '3': //compress dest
									if(chooser.showSaveDialog(frame)==JFileChooser.APPROVE_OPTION)
										compressDest.setText(chooser.getSelectedFile().getPath());
									break;
								case '4': //pack source
									chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
									if(chooser.showOpenDialog(frame)==JFileChooser.APPROVE_OPTION)
										packSource.setText(chooser.getSelectedFile().getPath());
									break;
								case '5': //pack dest
									if(chooser.showSaveDialog(frame)==JFileChooser.APPROVE_OPTION)
										packDest.setText(chooser.getSelectedFile().getPath());
									break;
							}
							break;
					}
				}
			}
		};
		
		t.start();
	}
	private void error(String error) {
		status(error);
		JOptionPane.showMessageDialog(frame, error, Common.NAME_STRING, JOptionPane.ERROR_MESSAGE);
	}
	private void status(String error) {
		status.setText(error);
		System.out.println(error);
	}
	
	public static void main(String[] args) {
		Common.startMessage();
		
		if(GraphicsEnvironment.isHeadless()) {
			System.out.println("No GUI layer found.");
			return;
		}
		
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			System.out.println("Failed to set look and feel to native.");
		}
		
		new GraphicalInterface();
	}
}
